package cn.com.qmth.examcloud.core.basic.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.api.commons.enums.BasicDataType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.core.basic.dao.SysConfigRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.SysConfigEntity;
import cn.com.qmth.examcloud.core.basic.service.SysConfigService;

/**
 * 系统配置服务
 *
 * @author WANGWEI
 * @date 2018年12月3日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class SysConfigServiceImpl implements SysConfigService {

	@Autowired
	SysConfigRepo sysConfigRepo;

	@Override
	public void set(String key, String value) {
		if (StringUtils.isBlank(key)) {
			throw new StatusException("350001", "key is blank");
		}

		SysConfigEntity sysConf = sysConfigRepo.getOne(key);
		if (null == sysConf) {
			throw new StatusException("350002", "key is wrong");
		}
		BasicDataType dataType = sysConf.getPropValueType();

		if (StringUtils.isBlank(value)) {
			sysConf.setPropValue(null);
		} else if (dataType.equals(BasicDataType.STRING)) {
			sysConf.setPropValue(value);
		} else if (dataType.equals(BasicDataType.LONG)) {
			try {
				sysConf.setPropValue(String.valueOf(Long.parseLong(value)));
			} catch (NumberFormatException e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.INTEGER)) {
			try {
				sysConf.setPropValue(String.valueOf(Integer.parseInt(value)));
			} catch (NumberFormatException e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.DATE)) {
			try {
				sysConf.setPropValue(
						DateUtil.format(DateUtil.parse(value, DatePatterns.CHINA_DEFAULT),
								DatePatterns.CHINA_DEFAULT));
			} catch (Exception e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.BOOLEAN)) {
			if ("true".equals(value)) {
				sysConf.setPropValue(value);
			} else if ("false".equals(value)) {
				sysConf.setPropValue(value);
			} else {
				throw new StatusException("350002", "value is wrong");
			}
		}

	}

	@Override
	public Object get(String key) {
		if (StringUtils.isBlank(key)) {
			throw new StatusException("350001", "key is blank");
		}

		SysConfigEntity sysConf = sysConfigRepo.getOne(key);
		if (null == sysConf) {
			throw new StatusException("350002", "key is wrong");
		}

		String value = sysConf.getPropValue();
		BasicDataType dataType = sysConf.getPropValueType();

		if (StringUtils.isBlank(value)) {
			return null;
		} else if (dataType.equals(BasicDataType.STRING)) {
			return value;
		} else if (dataType.equals(BasicDataType.LONG)) {
			try {
				return Long.parseLong(value);
			} catch (NumberFormatException e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.INTEGER)) {
			try {
				return Integer.parseInt(value);
			} catch (NumberFormatException e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.DATE)) {
			try {
				return DateUtil.parse(value, DatePatterns.CHINA_DEFAULT);
			} catch (Exception e) {
				throw new StatusException("350002", "value is wrong");
			}
		} else if (dataType.equals(BasicDataType.BOOLEAN)) {
			if ("true".equals(value)) {
				return true;
			} else if ("false".equals(value)) {
				return false;
			} else {
				throw new StatusException("350002", "value is wrong. value=" + value);
			}
		} else {
			throw new StatusException("350003", "dataType is undefined");
		}
	}

}
